package SPVM::Error::System;

1;

=head1 Name

SPVM::Error::System - System Error

=head1 Usage

  use Error::System;

=head1 Description

The Error::System class represents system errors.

=head1 Super Class

L<Error|SPVM::Error>

=head1 Copyright & License

Copyright (c) 2023 Yuki Kimoto

MIT License
