package SPVM::SPVM;

1;

=head1 Name

SPVM::SPVM - SPVM information

=head1 Description

SPVM class in L<SPVM> represents SPVM language.

A version string as same as the version of SPVM language is specified by C<version> statement.

=head1 Copyright & License

Copyright (c) 2023 Yuki Kimoto

MIT License
